<?php

/**
 * @file
 * Definition of WoW\RemoteEntityController.
 */

/**
 * This extends the EntityAPIController class, adding required special handling
 * for remote entity objects.
 *
 * To extends this controller, an entity must have the following requirements:
 *   - a 'lastFetched' field, corresponding to the timestamp it was last fetched
 *   from the service, not necessary updated, but at least fetched.
 *   - a refresh function of the form $entity_type . '_refresh' which is responsible
 *   to refresh an existing entity. This function should honor If-Modified-Since
 *   headers if needed, and saves the entity if updated.
 *   Important note: you need to set explicitely the original field before saving
 *   the entity, doing that will prevent an infinite loop between save and load
 *   mechanics.
 *   @see wow_character_refresh()
 *   @see wow_guild_refresh()
 */
abstract class WoWRemoteEntityController extends EntityAPIController {

  /**
   * Load an entity from the database.
   *
   * If the refresh method is set to loading time, calls the corresponding hook
   * responsible for refreshing an entity.
   */
  public function attachLoad(&$queried_entities, $revision_id = FALSE) {

    // Get the entity refresh method.
    if (wow_entity_refresh_method($this->entityType) == WOW_REFRESH_LOAD) {

      // Get the threshold value.
      $threshold = wow_entity_refresh_threshold($this->entityType);

      foreach ($queried_entities as $entity) {
          // Refresh the entity by calling the service.
        if ($entity->lastFetched + $threshold < REQUEST_TIME) {
          module_invoke($this->entityInfo['load hook'], 'refresh', $entity);
        }
      }
    }

    parent::attachLoad($queried_entities, $revision_id);
  }

  /**
   * Fetch a remote resource from the service and refresh it.
   *
   * @param WoWRemoteEntity $entity
   *   The entity to fetch.
   * @param array $query
   *   (Optional) An array of query key/value-pairs (without any URL-encoding) to
   *   append to the URL.
   *   - locale: You can specify your own locale here.
   *     It it the responsibility of the caller to pass a valid locale.
   *     @see wow_api_locale()
   * @param array $options
   *   (Optional) An array that can have one or more of the following elements:
   *   - headers: An array containing request headers to send as name/value pairs.
   *   - method: A string containing the request method. Defaults to 'GET'.
   *   - data: A string containing the request body, formatted as
   *     'param=value&param=value&...'. Defaults to NULL.
   *   - max_redirects: An integer representing how many times a redirect
   *     may be followed. Defaults to 3.
   *   - timeout: A float representing the maximum number of seconds the function
   *     call may take. The default is 30 seconds. If a timeout occurs, the error
   *     code is set to the HTTP_REQUEST_TIMEOUT constant.
   *   - context: A context resource created with stream_context_create().
   *
   * @return WoWRemoteEntity $entity
   *   The entity reference for chaining purpose.
   */
  public function fetch($entity, array $query = array(), array $options = array()) {
    // Executes the wow http request.
    $response = wow_http_request($entity->region, $entity->remoteURI(), $query, $options);

    // For efficiency manually set the original entity before applying any changes.
    $entity->original = clone $entity;

    // Updates the lastFetched timestamp with the request time.
    $entity->lastFetched = $response->getDate()->getTimestamp();

    if ($response instanceof WoWHttpResult) {

      // Merges the entity with the data from service.
      $entity->merge($response->getArray());

      // Let the controller subclasses hook the result.
      $this->processResult($entity, $response);

      // Saves the entity.
      $entity->save();
    }
    else {
      // Let the controller subclasses hook the status.
      $this->processStatus($entity, $response);
    }

    // Clean-up internals.
    unset($entity->original);

    return $entity;
  }

  /**
   * Processes a result from the service.
   *
   * @param WoWRemoteEntity $entity
   *   The entity being working on.
   * @param WoWHttpResult $result
   *   The http result.
   *
   * @see WoWGuildController
   * @see WoWCharacterController
   */
  abstract protected function processResult($entity, $result);

  /**
   * Processes a status from the service.
   *
   * @param WoWRemoteEntity $entity
   *   The entity being working on.
   * @param WoWHttpStatus $status
   *   The http status.
   *
   * @see WoWGuildController
   * @see WoWCharacterController
   */
  abstract protected function processStatus($entity, $status);
}
